program SQUIDcnf;

{$APPTYPE CONSOLE}
{$R DATA.RES}

uses
  Windows,
  SysUtils,
  ShellAPI,
  Classes;

const
  CAnon     = 'anon';
  CHelp     = 'help';
  CLocalDNS = 'localdns';
  CLogging  = 'logging';

function ConfigureSQUID : Boolean; forward;
function ExecuteProgram(const AFileName : String; const AParameters : String; const AWait : Boolean) : Boolean; forward;
function GetTimeString : String; forward;
function IsParameterSet(const AParameter : String) : Boolean; forward;
function SaveResourceIDToFile(const AResourceID : LongInt; const AFileName : String) : Boolean; forward;

function ConfigureSQUID : Boolean;
  function ReplaceSlash(const APath : String) : String;
  var
    LPos : LongInt;
  begin
    Result := APath;
    repeat
      LPos := Pos('\', Result);
      if (LPos > 0) then
        Result[LPos] := '/';
    until (LPos <= 0);
  end;
  
const
  CCacheLog   = 'var/logs/cache.log';
  CDefaultExt = '.default';
  CEtcFolder  = 'etc\';
  CMimeConf   = 'mime.conf';
  CMimePath   = 'etc/mime.conf';
  CPrgStart   = 'PrgStart.exe';
  CQuoteA     = 'http_access deny !Safe_ports';
  CQuoteB     = 'http_access deny CONNECT !SSL_ports';
  CSbinFolder = 'sbin\';
  CSearchLine = '# INSERT YOUR OWN RULE(S) HERE TO ALLOW ACCESS FROM YOUR CLIENTS';
  CSquidConf  = 'squid.conf';
  CSquidExe   = 'squid.exe';
  CStoreLog   = 'var/logs/store.log';
  CVarCache   = 'var/cache';
var
  LFound     : Boolean;
  LNextLine  : String;
  LReadFile  : TextFile;
  LSquidPath : String;
  LWriteFile : TextFile;
begin
  Result := false;

  LSquidPath := ExtractFilePath(ParamStr(0));
  if (ParamCount > 0) then
  begin
    if DirectoryExists(ParamStr(1)) then
    begin
      LSquidPath := Trim(ParamStr(1));
      if (LSquidPath[Length(LSquidPath)] <> '\') then
        LSquidPath := LSquidPath + '\';
    end;
  end;

  WriteLn('Searching etc-folder ...');
  if DirectoryExists(LSquidPath + CEtcFolder) then
  begin
    WriteLn('Searching mime.conf.default ...');
    if FileExists(LSquidPath + CEtcFolder + CMimeConf + CDefaultExt) then
    begin
      WriteLn('Searching mime.conf ...');
      if FileExists(LSquidPath + CEtcFolder + CMimeConf) then
      begin
        WriteLn('Renaming mime.conf ...');
        RenameFile(LSquidPath + CEtcFolder + CMimeConf,
                   LSquidPath + CEtcFolder + CMimeConf + '_' + GetTimeString);
      end;

      if not(FileExists(LSquidPath + CEtcFolder + CMimeConf)) then
      begin
        WriteLn('Copying mime.conf.default ...');
        AssignFile(LReadFile, LSquidPath + CEtcFolder + CMimeConf + CDefaultExt);
        Reset(LReadFile);
        try
          AssignFile(LWriteFile, LSquidPath + CEtcFolder + CMimeConf);
          Rewrite(LWriteFile);
          try
            while not(EoF(LReadFile)) do
            begin
              ReadLn(LReadFile, LNextLine);
              WriteLn(LWriteFile, LNextLine);
            end;
          finally
            CloseFile(LWriteFile);
          end;
        finally
          CloseFile(LReadFile);
        end;

        if FileExists(LSquidPath + CEtcFolder + CMimeConf) then
        begin
          WriteLn('Searching squid.conf.default ...');
          if FileExists(LSquidPath + CEtcFolder + CSquidConf + CDefaultExt) then
          begin
            WriteLn('Searching squid.conf ...');
            if FileExists(LSquidPath + CEtcFolder + CSquidConf) then
            begin
              WriteLn('Renaming squid.conf ...');
              RenameFile(LSquidPath + CEtcFolder + CSquidConf,
                         LSquidPath + CEtcFolder + CSquidConf + '_' + GetTimeString);
            end;

            if not(FileExists(LSquidPath + CEtcFolder + CSquidConf)) then
            begin
              WriteLn('Writing squid.conf ...');
              AssignFile(LReadFile, LSquidPath + CEtcFolder + CSquidConf + CDefaultExt);
              Reset(LReadFile);
              try
                AssignFile(LWriteFile, LSquidPath + CEtcFolder + CSquidConf);
                Rewrite(LWriteFile);
                try
                  LFound := false;
                  
                  while not(EoF(LReadFile)) do
                  begin
                    ReadLn(LReadFile, LNextLine);

                    if ((LNextLine = CQuoteA) or
                        (LNextLine = CQuoteB)) then
                      LNextLine := '# ' + LNextLine;
                    WriteLn(LWriteFile, LNextLine);

                    if not(LFound) then
                    begin
                      LFound := (LNextLine = CSearchLine);
                      if LFound then
                        WriteLn(LWriteFile, 'http_access allow localhost');
                    end;
                  end;

                  WriteLn(LWriteFile, '');
                  WriteLn(LWriteFile, '# SQUIDcnf configuration #');
                  WriteLn(LWriteFile, 'cache_mem 64 MB');
                  WriteLn(LWriteFile, 'cache_dir ufs ', ReplaceSlash(LSquidPath), CVarCache, ' 255 16 256');
                  WriteLn(LWriteFile, '');

                  if IsParameterSet(CLogging) then
                  begin
                    WriteLn(LWriteFile, 'cache_log ', ReplaceSlash(LSquidPath), CCacheLog);
                    WriteLn(LWriteFile, 'cache_store_log ', ReplaceSlash(LSquidPath), CStoreLog);
                  end;
                  
                  WriteLn(LWriteFile, '');

                  if IsParameterSet(CAnon) then
                    WriteLn(LWriteFile, 'forwarded_for off');                  

                  WriteLn(LWriteFile, 'via off');
                  WriteLn(LWriteFile, 'visible_hostname squid.localhost');
                  WriteLn(LWriteFile, '');

                  if IsParameterSet(CLocalDNS) then
                  begin
                    WriteLn(LWriteFile, 'dns_nameservers 127.0.0.1');
                    WriteLn(LWriteFile, '');
                  end;

                  WriteLn(LWriteFile, 'ftp_passive on');
                  WriteLn(LWriteFile, '');
                  WriteLn(LWriteFile, 'http_port 3128');
                  WriteLn(LWriteFile, '');
                  WriteLn(LWriteFile, 'mime_table ', ReplaceSlash(LSquidPath), CMimePath);
                finally
                  CloseFile(LWriteFile);
                end;
              finally
                CloseFile(LReadFile);
              end;

              if FileExists(LSquidPath + CEtcFolder + CSquidConf) then
              begin
                WriteLn('Searching sbin-folder ...');
                if DirectoryExists(LSquidPath + CSbinFolder) then
                begin
                  WriteLn('Searching squid.exe ...');
                  if FileExists(LSquidPath + CSbinFolder + CSquidExe) then
                  begin
                    WriteLn('Generating cache ...');
                    ExecuteProgram(LSquidPath + CSbinFolder + CSquidExe,
                                   '-z', true);

                    WriteLn('Extracting SQUID starter ...');
                    if SaveResourceIDToFile(100, LSquidPath + CSbinFolder + CPrgStart) then
                    begin
                      WriteLn('Starting SQUID starter ...');

                      Result := ExecuteProgram(LSquidPath + CSbinFolder + CPrgStart,
                                               '"app=' + LSquidPath + CSbinFolder + CSquidExe + '" ' +
                                               '"name=SQUID" "wait=15000"', false);
                      if not(Result) then
                        WriteLn('SQUID starter not started!');
                    end
                    else
                      WriteLn('SQUID starter not extracted!');
                  end
                  else
                    WriteLn('squid.exe not found!');
                end
                else
                  WriteLn('sbin-folder not found!');
              end
              else
                WriteLn('squid.conf not written!');
            end
            else
              WriteLn('squid.conf not renamed!');
          end
          else
            WriteLn('squid.conf.default not found!');
        end
        else
          WriteLn('mime.conf.default not copied!');
      end
      else
        WriteLn('mime.conf not renamed!');
    end
    else
      WriteLn('mime.conf.default not found!');
  end
  else
    WriteLn('etc-folder not found!');
end;

function ExecuteProgram(const AFileName : String; const AParameters : String; const AWait : Boolean) : Boolean;
var
   LExitCode : LongWord;
   LSEInfo   : TShellExecuteInfo;
begin
  Result := false;

  if FileExists(AFileName) then
  begin
    FillChar(LSEInfo, SizeOf(LSEInfo), 0);
    LSEInfo.cbSize       := SizeOf(TShellExecuteInfo);
    LSEInfo.fMask        := SEE_MASK_NOCLOSEPROCESS;
    LSEInfo.lpDirectory  := PChar(ExtractFilePath(AFileName));
    LSEInfo.lpFile       := PChar(AFileName);
    LSEInfo.lpParameters := PChar(AParameters);
    LSEInfo.nShow        := SW_SHOWNORMAL;
    LSEInfo.Wnd          := 0;

    if ShellExecuteEx(@LSEInfo) then
    begin
      if AWait then
      begin
        repeat
          Sleep(100);

          GetExitCodeProcess(LSEInfo.hProcess, LExitCode);
        until (LExitCode <> STILL_ACTIVE);
      end;

      Result := true;
    end;
  end;
end;

function GetTimeString : String;
  function ToString(const ANum : Word; const ALength : Byte) : String;
  begin
    Result := IntToStr(ANum);
    while (Length(Result) < ALength) do
      Result := '0' + Result;
  end;
var
  LDay     : Word;
  LHour    : Word;
  LMinute  : Word;
  LMonth   : Word;
  LMSecond : Word;
  LSecond  : Word;
  LYear    : Word;
begin
  DecodeDate(Now, LYear, LMonth, LDay);
  DecodeTime(Now, LHour, LMinute, LSecond, LMSecond);

  Result := ToString(LYear, 4) + ToString(LMonth, 2) + ToString(LDay, 2) +
            ToString(LHour, 2) + ToString(LMinute, 2) + ToString(LSecond, 2);
end;

function IsParameterSet(const AParameter : String) : Boolean;
var
  LIndex : LongInt;
begin
  Result := false;

  for LIndex := 1 to ParamCount do
  begin
    Result := (AnsiLowerCase(ParamStr(LIndex)) = AnsiLowerCase(AParameter));
    if Result then
      Break;
  end
end;

function SaveResourceIDToFile(const AResourceID : LongInt; const AFileName : String) : Boolean;
var
  LResourceStream : TResourceStream;
begin
  LResourceStream := TResourceStream.CreateFromID(hInstance, AResourceID, RT_RCDATA);
  try
    LResourceStream.SaveToFile(AFileName);
  finally
    LResourceStream.Free;
  end;

  Result := FileExists(AFileName);
end;

begin
  WriteLn('');
  WriteLn('SQUIDcnf 0.1b2');
  WriteLn('(C) 2009 coltishWARE');
  WriteLn('');

  if IsParameterSet(CHelp) then
  begin
    WriteLn(ExtractFileName(ParamStr(0)), ' [SQUIDPATH] [', CAnon, '] [', CHelp,
            '] [', CLocalDNS, '] [', CLogging, ']');
    WriteLn('');
    WriteLn('SQUIDPATH    path to the SQUID installation');
    WriteLn('             current directory is used if left blank');
    WriteLn('');
    WriteLn(CAnon, '         deactivate "forwarded for" info');
    WriteLn(CHelp, '         show this help');
    WriteLn(CLocalDNS, '     sets local DNS server as primary');
    WriteLn(CLogging, '      activates logging');
  end
  else
  begin
    if ConfigureSQUID then
    begin
      WriteLn('');
      WriteLn('CONFIGURATION SUCCEEDED');
    end
    else
    begin
      WriteLn('');
      WriteLn('CONFIGURATION FAILED');
    end;
  end;

  WriteLn('');
end.
